package language;

import com.intellij.lang.ASTNode;
import com.intellij.psi.PsiComment;
import com.intellij.psi.PsiElement;
import com.intellij.psi.util.PsiTreeUtil;
import org.jetbrains.yaml.YAMLTokenTypes;
import org.jetbrains.yaml.psi.YAMLDocument;
import org.jetbrains.yaml.psi.YAMLScalarList;

/**
 * Created with IntelliJ IDEA.
 * User: Marcin Rembała
 * Date: 16.04.14
 */
public final class YamlUtils {

    public static final String FIELD_SCRIPT = "script";

    public static int getIndentForCurrentElement(PsiElement current) {
        current = adjustCurrentElement(current);
        ASTNode node = current.getNode();
        if (node != null && node.getElementType() == YAMLTokenTypes.EOL) {
            return getIndent(current.getPrevSibling());
        } else {
            return getIndent(current);
        }
    }

    private static PsiElement adjustCurrentElement(PsiElement current) {
        if (current instanceof YAMLScalarList) {
            PsiElement temp = current.getLastChild();
            while (temp != null && (temp instanceof PsiComment || temp.getNode().getElementType() == YAMLTokenTypes.EOL)) {
                temp = PsiTreeUtil.prevLeaf(temp);
            }
            current = temp != null ? temp : current.getLastChild();
        }
        return current;
    }

    public static PsiElement getPrevOrParent(final PsiElement current) {
        final PsiElement prevSibling = current.getPrevSibling();
        return prevSibling != null ? prevSibling : current.getParent();
    }

    public static int getIndent(final PsiElement current) {
        PsiElement temp = current;
        while (!(temp instanceof YAMLDocument) && !(temp == null || temp.getNode() == null || YAMLTokenTypes.EOL == temp.getNode().getElementType())) {
            if (YAMLTokenTypes.INDENT == temp.getNode().getElementType()) {
                return temp.getTextLength();
            }
            temp = getPrevOrParent(temp);
        }
        return 0;
    }

    public static int getScriptBlockLength(PsiElement value) {
        PsiElement temp = value.getFirstChild();
        if (temp != null) {
            while ((temp = temp.getNextSibling()) != null && temp.getParent() == value) {
                if (temp.getNode().getElementType() == YAMLTokenTypes.COMMENT) {
                    return temp.getStartOffsetInParent();
                }
            }
        }
        return value.getTextLength();
    }

}
