package language.injection;

import com.intellij.lang.injection.MultiHostInjector;
import com.intellij.lang.injection.MultiHostRegistrar;
import com.intellij.openapi.util.Key;
import com.intellij.openapi.util.TextRange;
import com.intellij.openapi.util.Trinity;
import com.intellij.psi.PsiElement;
import com.intellij.psi.PsiLanguageInjectionHost;
import language.YamlUtils;
import org.intellij.plugins.intelliLang.inject.InjectedLanguage;
import org.intellij.plugins.intelliLang.inject.InjectorUtils;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.plugins.groovy.GroovyFileType;
import org.jetbrains.yaml.psi.YAMLKeyValue;
import org.jetbrains.yaml.psi.YAMLScalarList;

import java.util.Collections;
import java.util.List;

/**
 * Created with IntelliJ IDEA.
 * User: Marcin Rembała
 * Date: 18.02.15
 */
public class GroovyToYamlScriptInjector implements MultiHostInjector {

    public static final Key<YamlScriptBlockInjectionHost> INJECTION_HOST_KEY = Key.create("nmo.yaml.script.host");

    @Override
    public void getLanguagesToInject(@NotNull MultiHostRegistrar registrar, @NotNull PsiElement context) {
        assert context instanceof YAMLScalarList;
        final YAMLScalarList scalarList = (YAMLScalarList) context;
        final PsiElement parent = scalarList.getParent();
        if (parent instanceof YAMLKeyValue) {
            final YAMLKeyValue keyValue = (YAMLKeyValue) parent;
            if (YamlUtils.FIELD_SCRIPT.equals(keyValue.getKeyText())) {
                final String prefix = getContextPrefix(keyValue);
                final String suffix = getContextSuffix(keyValue);
                final InjectedLanguage injectedLanguage = InjectedLanguage.create(GroovyFileType.GROOVY_LANGUAGE.getID(), prefix, suffix, true);
                final YamlScriptBlockInjectionHost injectionHost = getInjectionHost(scalarList);
                injectionHost.setPrefix(prefix);
                final Trinity<PsiLanguageInjectionHost, InjectedLanguage, TextRange> injectionTrinity = Trinity.create((PsiLanguageInjectionHost) injectionHost, injectedLanguage, TextRange.from(1, YamlUtils.getScriptBlockLength(context) - 1));
                InjectorUtils.registerInjection(GroovyFileType.GROOVY_LANGUAGE, Collections.singletonList(injectionTrinity), context.getContainingFile(), registrar);
            }
        }
    }

    private String getContextPrefix(YAMLKeyValue scriptKeyValue) {
        return "//my prefix";
    }

    private String getContextSuffix(YAMLKeyValue scriptKeyValue) {
        return null;
    }

    private YamlScriptBlockInjectionHost getInjectionHost(YAMLScalarList scalarList) {
        YamlScriptBlockInjectionHost injectionHost = scalarList.getUserData(INJECTION_HOST_KEY);
        if (injectionHost == null) {
            injectionHost = new YamlScriptBlockInjectionHost(scalarList);
            scalarList.putUserData(INJECTION_HOST_KEY, injectionHost);
        }
        return injectionHost;
    }

    @NotNull
    @Override
    public List<? extends Class<? extends PsiElement>> elementsToInjectIn() {
        return Collections.singletonList(YAMLScalarList.class);
    }

}
