package language.injection;

import com.intellij.openapi.components.AbstractProjectComponent;
import com.intellij.openapi.project.Project;
import com.intellij.psi.PsiFileFactory;
import com.intellij.psi.util.PsiTreeUtil;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.plugins.groovy.GroovyFileType;
import org.jetbrains.yaml.YAMLFileType;
import org.jetbrains.yaml.psi.YAMLScalarList;
import org.jetbrains.yaml.psi.impl.YAMLFileImpl;

/**
 * Created with IntelliJ IDEA.
 * User: Marcin Rembała
 * Date: 19.02.15
 */
public class YAMLPsiElementFactory extends AbstractProjectComponent {
    @NonNls
    public static final String DUMMY_FILE_NAME = "YAML_DUMMY__1234567890_DUMMYYYYYY___";

    public YAMLPsiElementFactory(Project project) {
        super(project);
    }

    @NotNull
    public YAMLScalarList createScalarListFromText(@NotNull String text) {
        final String scalarListText = "dummyProperty: |" + text + "\n#";
        final YAMLFileImpl dummyFile = createDummyFile(scalarListText, false);
        //noinspection ConstantConditions
        return PsiTreeUtil.findChildOfType(dummyFile, YAMLScalarList.class);
    }

    private YAMLFileImpl createDummyFile(CharSequence text, boolean physical) {
        final String fileName = DUMMY_FILE_NAME + '.' + GroovyFileType.GROOVY_FILE_TYPE.getDefaultExtension();
        final PsiFileFactory factory = PsiFileFactory.getInstance(myProject);
        return (YAMLFileImpl) factory.createFileFromText(fileName, YAMLFileType.YML, text, System.currentTimeMillis(), physical);
    }
}
