package language.injection;

import com.intellij.lang.ASTNode;
import com.intellij.lang.Language;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.util.Key;
import com.intellij.openapi.util.TextRange;
import com.intellij.psi.*;
import com.intellij.psi.scope.PsiScopeProcessor;
import com.intellij.psi.search.GlobalSearchScope;
import com.intellij.psi.search.SearchScope;
import com.intellij.util.IncorrectOperationException;
import language.YamlUtils;
import org.apache.commons.lang.StringUtils;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.jetbrains.yaml.psi.YAMLScalarList;

import javax.swing.*;

/**
 * Created with IntelliJ IDEA.
 * User: Marcin Rembała
 * Date: 18.02.15
 */
@SuppressWarnings("deprecation")
public class YamlScriptBlockInjectionHost implements PsiLanguageInjectionHost {
    @NotNull
    private final YAMLScalarList context;
    private String prefix = null;

    public YamlScriptBlockInjectionHost(@NotNull YAMLScalarList context) {
        this.context = context;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof YAMLScalarList) {
            return context.equals(obj);
        } else {
            return super.equals(obj);
        }
    }

    @Override
    @NotNull
    public Project getProject() throws PsiInvalidElementAccessException {
        return context.getProject();
    }

    @Override
    @NotNull
    public Language getLanguage() {
        return context.getLanguage();
    }

    @Override
    public PsiManager getManager() {
        return context.getManager();
    }

    @Override
    @NotNull
    public PsiElement[] getChildren() {
        return context.getChildren();
    }

    @Override
    public PsiElement getParent() {
        return context.getParent();
    }

    @Override
    public PsiElement getFirstChild() {
        return context.getFirstChild();
    }

    @Override
    public PsiElement getLastChild() {
        return context.getLastChild();
    }

    @Override
    @Nullable
    public PsiElement getNextSibling() {
        return context.getNextSibling();
    }

    @Override
    @Nullable
    public PsiElement getPrevSibling() {
        return context.getPrevSibling();
    }

    @Override
    public PsiFile getContainingFile() throws PsiInvalidElementAccessException {
        return context.getContainingFile();
    }

    @Override
    public TextRange getTextRange() {
        return context.getTextRange();
    }

    @Override
    public int getStartOffsetInParent() {
        return context.getStartOffsetInParent();
    }

    @Override
    public int getTextLength() {
        return context.getTextLength();
    }

    @Override
    @Nullable
    public PsiElement findElementAt(int offset) {
        return context.findElementAt(offset);
    }

    @Override
    @Nullable
    public PsiReference findReferenceAt(int offset) {
        return context.findReferenceAt(offset);
    }

    @Override
    public int getTextOffset() {
        return context.getTextOffset();
    }

    @Override
    @NonNls
    public String getText() {
        return context.getText();
    }

    @Override
    @NotNull
    public char[] textToCharArray() {
        return context.textToCharArray();
    }

    @Override
    public PsiElement getNavigationElement() {
        return context.getNavigationElement();
    }

    @Override
    public PsiElement getOriginalElement() {
        return context.getOriginalElement();
    }

    @Override
    public boolean textMatches(@NotNull @NonNls CharSequence text) {
        return context.textMatches(text);
    }

    @Override
    public boolean textMatches(@NotNull PsiElement element) {
        return context.textMatches(element);
    }

    @Override
    public boolean textContains(char c) {
        return context.textContains(c);
    }

    @Override
    public void accept(@NotNull PsiElementVisitor visitor) {
        context.accept(visitor);
    }

    @Override
    public void acceptChildren(@NotNull PsiElementVisitor visitor) {
        context.acceptChildren(visitor);
    }

    @Override
    public PsiElement copy() {
        return context.copy();
    }

    @Override
    public PsiElement add(@NotNull PsiElement element) throws IncorrectOperationException {
        return context.add(element);
    }

    @Override
    public PsiElement addBefore(@NotNull PsiElement element, @Nullable PsiElement anchor) throws IncorrectOperationException {
        return context.addBefore(element, anchor);
    }

    @Override
    public PsiElement addAfter(@NotNull PsiElement element, @Nullable PsiElement anchor) throws IncorrectOperationException {
        return context.addAfter(element, anchor);
    }

    @Override
    public void checkAdd(@NotNull PsiElement element) throws IncorrectOperationException {
        context.checkAdd(element);
    }

    @Override
    public PsiElement addRange(PsiElement first, PsiElement last) throws IncorrectOperationException {
        return context.addRange(first, last);
    }

    @Override
    public PsiElement addRangeBefore(@NotNull PsiElement first, @NotNull PsiElement last, PsiElement anchor) throws IncorrectOperationException {
        return context.addRangeBefore(first, last, anchor);
    }

    @Override
    public PsiElement addRangeAfter(PsiElement first, PsiElement last, PsiElement anchor) throws IncorrectOperationException {
        return context.addRangeAfter(first, last, anchor);
    }

    @Override
    public void delete() throws IncorrectOperationException {
        context.delete();
    }

    @Override
    public void checkDelete() throws IncorrectOperationException {
        context.checkDelete();
    }

    @Override
    public void deleteChildRange(PsiElement first, PsiElement last) throws IncorrectOperationException {
        context.deleteChildRange(first, last);
    }

    @Override
    public PsiElement replace(@NotNull PsiElement newElement) throws IncorrectOperationException {
        return context.replace(newElement);
    }

    @Override
    public boolean isValid() {
        return context.isValid();
    }

    @Override
    public boolean isWritable() {
        return context.isWritable();
    }

    @Override
    @Nullable
    public PsiReference getReference() {
        return context.getReference();
    }

    @Override
    @NotNull
    public PsiReference[] getReferences() {
        return context.getReferences();
    }

    @Override
    @Nullable
    public <T> T getCopyableUserData(Key<T> key) {
        return context.getCopyableUserData(key);
    }

    @Override
    public <T> void putCopyableUserData(Key<T> key, @Nullable T value) {
        context.putCopyableUserData(key, value);
    }

    @Override
    public boolean processDeclarations(@NotNull PsiScopeProcessor processor, @NotNull ResolveState state, @Nullable PsiElement lastParent, @NotNull PsiElement place) {
        return context.processDeclarations(processor, state, lastParent, place);
    }

    @Override
    @Nullable
    public PsiElement getContext() {
        return context.getContext();
    }

    @Override
    public boolean isPhysical() {
        return context.isPhysical();
    }

    @Override
    @NotNull
    public GlobalSearchScope getResolveScope() {
        return context.getResolveScope();
    }

    @Override
    @NotNull
    public SearchScope getUseScope() {
        return context.getUseScope();
    }

    @Override
    public ASTNode getNode() {
        return context.getNode();
    }

    @Override
    @NonNls
    public String toString() {
        return context.toString();
    }

    @Override
    public boolean isEquivalentTo(PsiElement another) {
        return context.isEquivalentTo(another);
    }

    @Override
    @Nullable
    public <T> T getUserData(@NotNull Key<T> key) {
        return context.getUserData(key);
    }

    @Override
    public <T> void putUserData(@NotNull Key<T> key, @Nullable T value) {
        context.putUserData(key, value);
    }

    @Override
    public Icon getIcon(@IconFlags int flags) {
        return context.getIcon(flags);
    }

    @Override
    public boolean isValidHost() {
        return context.isValid();
    }

    @Override
    public PsiLanguageInjectionHost updateText(@NotNull String text) {
        final int indent = YamlUtils.getIndentForCurrentElement(context);
        text = prepareNewText(text, indent);
        final ASTNode currentNode = context.getNode();
        final YAMLScalarList newScalarList = context.getProject().getComponent(YAMLPsiElementFactory.class).createScalarListFromText(text);
        currentNode.replaceAllChildrenToChildrenOf(newScalarList.getNode());
        return this;
    }

    private String prepareNewText(String text, int indent) {
        //not required for IU-13
        /*final String currentPrefix = prefix;
        if (currentPrefix != null) {
            //cutoff injection prefix new host text - imports are appended before this prefix
            text = text.replace(currentPrefix, StringUtils.EMPTY);
            text = text.replaceAll("\\n(\\S)", "\n" + StringUtils.repeat(" ", indent) + "$1");
        }*/
        if (text.startsWith("|\n")) {
            text = text.substring(1);
        } else if (text.startsWith("|")) {
            text = text.replaceFirst("\\|", "\n" + StringUtils.repeat(" ", indent));
        }
        return text;
    }

    @NotNull
    @Override
    public LiteralTextEscaper<? extends PsiLanguageInjectionHost> createLiteralTextEscaper() {
        return new ScriptBlockLiteralTextEscaper(this);
    }

    public void setPrefix(String prefix) {
        this.prefix = prefix;
    }
}
